"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseClient = void 0;
const got_1 = __importDefault(require("got"));
const querystring_1 = __importDefault(require("querystring"));
const formatted_error_utils_1 = require("../utils/formatted-error.utils");
const SUPPORTS_BODY = ['post', 'put', 'patch'];
class BaseClient {
    constructor(baseUrl, options) {
        this._supportedMethods = {};
        this.api = got_1.default;
        this._baseUrl = baseUrl;
        this._pathPrefix = options.pathPrefix;
        this._commonHeaders = options.headers;
        this._fillSupportedMethods();
    }
    get baseUrl() { return this._baseUrl; }
    apiCall(options, callback) {
        this._checkArgs(options);
        const formattedQs = options.qs ? `?${querystring_1.default.stringify(options.qs)}` : '';
        const authHeaders = this._commonHeaders ?
            Object.assign(this._commonHeaders, options.headers) : options.headers;
        const gotOptions = {
            responseType: "json",
            headers: Object.assign(this._extendedHeaders({ body: options.body, qs: options.qs }), authHeaders),
            json: options.body ? options.body : undefined
        };
        this._fillSupportedMethods();
        return this._supportedMethods[options.method](`${this._baseUrl}${this._pathPrefix}${options.path}${formattedQs}`, gotOptions).then(response => {
            if (response.statusCode !== 200) {
                return callback(new Error('Status is not 200'));
            }
            if (response.body !== null && response.body !== undefined &&
                typeof response.body === 'object' &&
                response.body.hasOwnProperty('status') &&
                response.body.status !== 200) {
                return callback(response.body);
            }
            return callback(null, response.body);
        }).catch(error => {
            // console.log(error);
            // console.log(error.response.body);
            callback(error);
        });
    }
    _fillSupportedMethods() {
        this._supportedMethods = {
            'get': this.api.get,
            'head': this.api.head, 'post': this.api.post,
            'put': this.api.put, 'patch': this.api.patch,
            'delete': this.api.delete
        };
    }
    _checkArgs(options) {
        if (!this._supportedMethods[options.method])
            throw new formatted_error_utils_1.FormattedError('`method` argument must be one of `%j`, got `%j`', this._supportedMethods, options.method);
        if ((typeof options.path !== 'string') || (options.path.length < 1))
            throw new formatted_error_utils_1.FormattedError('`path` argument must be non-empty string, got `%j`', options.path);
        if (!SUPPORTS_BODY.includes(options.method) && (options.body !== null))
            throw new formatted_error_utils_1.FormattedError('`%s` does not support body: `%j` passed in, expected `null` (use `qs` param instead)', options.method, options.body);
    }
    _extendedHeaders(options) {
        const reqId = (options.qs || {}).req_id || (options.body || {}).req_id || null;
        return reqId
            ? { 'x-request-id': reqId }
            : {};
    }
}
exports.BaseClient = BaseClient;
