"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomersClient = void 0;
const config_1 = require("../../../config");
const BaseClient_abstract_1 = require("../../common/BaseClient.abstract");
const helper_1 = require("../../utils/helper");
/**
 * class CustomersClient
 * constructor requires options as
 * {
 *  protocol: 'http',
 *  hostname: 'domain.com',
 *  port: 80,
 *  pathnamePrefix?: '/v2/customers',
 *  clientId?: 'my-client-id',
 *  secret?: 'secret key if exists',
 *  agent?: string
 * }
 */
class CustomersClient extends BaseClient_abstract_1.BaseClient {
    /**
     * constructor
     *
     * @param options ClientOptions
     */
    constructor(options) {
        super(options.url ? options.url.toString() : config_1.config.url, {
            pathPrefix: config_1.config.V2.pathPrefix,
            headers: { Authorization: `Basic ${(0, helper_1.base64)(options.appName + ':' + options.secretKey)}` }
        });
    }
    /**
     * Get customer purchases
     *
     * @param applicationUsername - string application-username
     * @param cb - callback method for getting the data.
     * @returns - void
     */
    getCustomerPurchases(applicationUsername, cb) {
        if (applicationUsername === null || applicationUsername === '' || applicationUsername === undefined) {
            return cb(new Error("UserId parameter is required to be non-null and non-empty"));
        }
        if (typeof applicationUsername !== 'string')
            return cb(new Error("applicationUsername parameter should be a string"));
        this.apiCall({
            method: 'get', path: `/${applicationUsername}/purchases`,
            qs: null, headers: {}, body: null
        }, (err, result) => {
            if (err) {
                return cb(err);
            }
            cb(null, result);
        });
    }
    /**
     * Get bulk information about your customers.
     *
     * @param params - GetCustomersBulkInfoParams => limit, skip, applicationUsername
     * @param cb - callback method for getting the data.
     */
    getCustomersBulkInfo(params, cb) {
        const isApplicationUsersExist = (variableToCheck) => variableToCheck.applicationUsername !== undefined;
        const isPagingParam = (variableToCheck) => variableToCheck.limit !== undefined;
        const buildBulkQuerystring = (params) => {
            var _a, _b;
            if (isApplicationUsersExist(params) && ((_a = params.applicationUsername) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                return {
                    applicationUsername: (_b = params.applicationUsername) === null || _b === void 0 ? void 0 : _b.map((name, i) => encodeURIComponent(name)).join(',')
                };
            }
            else if (isPagingParam(params) && params.limit > 0) {
                params.skip = params.skip || 0;
                return params;
            }
            return null;
        };
        const qs = buildBulkQuerystring(params);
        if (qs === null) {
            return cb(new Error("skip and limit are disregarded when 'applicationUsername' parameter" +
                " is specified, else skip and limit should be defined."));
        }
        this.apiCall({
            method: 'get', path: '/',
            qs: qs, headers: {}, body: null
        }, (err, result) => {
            if (err) {
                return cb(err);
            }
            cb(null, result);
        });
    }
}
exports.CustomersClient = CustomersClient;
